#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2022/12/8 21:21
# @Author : xiang
# @File : dicts
# @Project : caishengxiang
"""
描述：各种字典
"""
from collections import OrderedDict
from collections import defaultdict


def ordereddict_example():
    """
    排序字典
    OrderedDict 内部维护着一个根据键插入顺序排序的双向链表。
每次当一个新的元素插入进来的时候， 它会被放到链表的尾部。
对于一个已经存在的键的重复赋值不会改变键的顺序。

需要注意的是，一个 OrderedDict 的大小是一个普通字典的两倍，
因为它内部维护着另外一个链表。
所以如果你要构建一个需要大量 OrderedDict 实例的数据结构的时候
（比如读取 100,000 行 CSV 数据到一个 OrderedDict 列表中去），
那么你就得仔细权衡一下是否使用 OrderedDict 带来的好处要大过额外内存消耗的影响。
    :return:
    """
    d = OrderedDict()
    d['foo'] = 1
    d['bar'] = 2
    d['spam'] = 3
    d['grok'] = 4
    # Outputs "foo 1", "bar 2", "spam 3", "grok 4"
    print(d)
    for key in d:
        print(key, d[key])


def defaultdict_example():
    """
    默认值字典
    defaultdict是Python内建dict类的一个子类，第一个参数为default_factory属性提供初始值，默认为None。
    它覆盖一个方法并添加一个可写实例变量。它的其他功能与dict相同，但会为一个不存在的键提供默认值，从而避免KeyError异常。
    :return:
    """
    def default_value():
        return '默认值'

    a = defaultdict(list)
    a['a'].append(1)
    a['a'].append(2)
    a['b'].append(4)

    d = defaultdict(set)
    d['a'].add(1)
    d['a'].add(2)
    d['b'].add(4)

    print(a)
    print(d)

    e = defaultdict(default_value)
    print(e['zz'])
    print(e)




if __name__ == '__main__':
    ordereddict_example()
    defaultdict_example()



from collections import namedtuple
import struct


# zip constants
ZIP32_VERSION = 20
ZIP64_VERSION = 45
ZIP32_LIMIT = (1 << 31) - 1
UTF8_FLAG = 0x800   # utf-8 filename encoding flag

# zip compression methods
COMPRESSION_STORE = 0
COMPRESSION_DEFLATE = 8
COMPRESSION_BZIP2 = 12
COMPRESSION_LZMA = 14

# file header
LF_STRUCT = struct.Struct(b"<4sHHHHHLLLHH")
LF_TUPLE = namedtuple("fileheader",
                      ("signature", "version", "flags",
                       "compression", "mod_time", "mod_date",
                       "crc", "comp_size", "uncomp_size",
                       "fname_len", "extra_len"))
LF_MAGIC = b'\x50\x4b\x03\x04'

# extra fields
EXTRA_STRUCT = struct.Struct(b"<HH")
EXTRA_TUPLE = namedtuple("extra", ("signature", "size"))
EXTRA_64_STRUCT = struct.Struct(b"<HH")
EXTRA_64_TUPLE = namedtuple("extra64local", ("uncomp_size", "comp_size"))
CD_EXTRA_64_STRUCT = struct.Struct(b"<HHH")
CD_EXTRA_64_TUPLE = namedtuple(
    "extra64cdir", ("uncomp_size", "comp_size", "offset"))

# file descriptor
DD_STRUCT = struct.Struct(b"<LLL")
DD_STRUCT64 = struct.Struct(b"<LQQ")
DD_TUPLE = namedtuple("filecrc",
                      ("crc", "comp_size", "uncomp_size"))
DD_MAGIC = b'\x50\x4b\x07\x08'

# central directory file header
CDLF_STRUCT = struct.Struct(b"<4sBBHHHHHLLLHHHHHLL")
CDLF_TUPLE = namedtuple("cdfileheader",
                        ("signature", "system", "version", "version_ndd", "flags",
                         "compression", "mod_time", "mod_date", "crc",
                         "comp_size", "uncomp_size", "fname_len", "extra_len",
                         "fcomm_len", "disk_start", "attrs_int", "attrs_ext", "offset"))
CDFH_MAGIC = b'\x50\x4b\x01\x02'

# end of central directory record
CD_END_STRUCT = struct.Struct(b"<4sHHHHLLH")
CD_END_TUPLE = namedtuple("cdend",
                          ("signature", "disk_num", "disk_cdstart", "disk_entries",
                           "total_entries", "cd_size", "cd_offset", "comment_len"))
CD_END_MAGIC = b'\x50\x4b\x05\x06'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/7/19 上午12:32
# @Author  : caishengxiang
# @File    : chmod.py
"""
权限管理
"""
import os
import stat
import json
import pwd


def get_chmod_number(path: str) -> str:
    """获取文件/文件夹 权限"""
    s = os.stat(path)
    mode = s[stat.ST_MODE]
    usr_num = 0
    if mode & stat.S_IRUSR > 0:
        usr_num += 4
    if mode & stat.S_IWUSR > 0:
        usr_num += 2
    if mode & stat.S_IXUSR > 0:
        usr_num += 1
    grp_num = 0
    if mode & stat.S_IRGRP > 0:
        grp_num += 4
    if mode & stat.S_IWGRP > 0:
        grp_num += 2
    if mode & stat.S_IXGRP > 0:
        grp_num += 1

    oth_num = 0
    if mode & stat.S_IROTH > 0:
        oth_num += 4
    if mode & stat.S_IWOTH > 0:
        oth_num += 2
    if mode & stat.S_IXOTH > 0:
        oth_num += 1
    chmod_num = str(usr_num) + str(grp_num) + str(oth_num)
    return chmod_num



# ---encoding:utf-8---
# @Time    : 2023/8/1 21:29
# @Author  : caishengxiang
# @Email   : 383301857@qq.com
# @File    : echarts_draw.py
# @Project : caishengxiang
try:
    from pyecharts.charts import Line
    from pyecharts import options as opts
except:
    raise Exception('you need: pip install pyecharts')


class Draw:
    def __init__(self, y_name, x_name, title='draw', save_path='./draw.html'):
        self.y_name = y_name
        self.x_name = x_name
        self.title = title
        self.save_path = save_path
        self.x_datas = []
        self.y_datas = []

    def add_data(self, x_data, y_data):
        # 添加 x 轴数据

        self.x_datas.append(x_data)

        # 添加 y 轴数据
        self.y_datas.append(y_data)

    def draw(self):
        line_chart = Line()
        line_chart.add_xaxis(self.x_datas)
        line_chart.add_yaxis(self.y_name, self.y_datas)

        # 设置图表标题和 x 轴、y 轴标签
        line_chart.set_global_opts(title_opts=opts.TitleOpts(title=self.title),
                                   xaxis_opts=opts.AxisOpts(name=self.x_name),
                                   yaxis_opts=opts.AxisOpts(name=self.y_name))
        return line_chart

    def save(self):
        line_chart = self.draw()
        # 将图表保存为 HTML 格式的文件
        line_chart.render(self.save_path)


if __name__ == '__main__':
    from pyecharts.charts import Grid,Page

    grid = Grid()
    d = Draw(y_name='1使用率%', x_name='时间', title='{}: cpu监控'.format('localhost'),
             save_path='./cpu.html')
    d.add_data(1, 2)
    d.add_data(2, 2)
    d.add_data(3, 2)
    d.add_data(3, 2)
    d.add_data(3, 2)
    d.add_data(3, 2)

    d2 = Draw(y_name='2使用率%', x_name='时间', title='{}: cpu监控'.format('localhost'),
             save_path='./cpu.html')
    d2.add_data(1, 2)
    d2.add_data(2, 2)
    d2.add_data(2, 2)
    d2.add_data(2, 2)
    d2.add_data(4, 2)

    d3 = Draw(y_name='3使用率%', x_name='时间', title='{}: cpu监控'.format('localhost'),
             save_path='./cpu.html')
    d3.add_data(1, 2)
    d3.add_data(2, 2)
    d3.add_data(2, 2)
    d3.add_data(2, 2)
    d3.add_data(2, 2)
    d3.add_data(4, 2)

    page=Page()
    page.add(d.draw())
    page.add(d2.draw())
    page.add(d3.draw())
    page.render('./cpu.html')
    # grid_chart = Grid()
    # grid_chart.add(d.draw(), grid_opts=opts.GridOpts(pos_top="0%", pos_bottom='70%'))
    # grid_chart.add(d2.draw(), grid_opts=opts.GridOpts(pos_top="30%", pos_bottom='40%'))
    # grid_chart.add(d3.draw(), grid_opts=opts.GridOpts(pos_top="60%"))
    # grid_chart.render('./cpu.html')
    # d.save()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2022/12/8 0:32
# @Author : xiang
# @File : c_date
# @Project : caishengxiang
"""
描述：
"""
from datetime import datetime, timedelta,date
import calendar


def get_previous_day_by_week(dayname, end_date=None):
    """
    查找最后一个星期五的日期。
    算法：
    先将开始日期和目标日期映射到星期数组的位置上(星期一索引为0)， 然后通过模运算计算出目标日期要经过多少天才能到达开始日期。然后用开始日期减去那个时间差即得到结果日期。
    :param dayname:
    :param end_date:
    :return:
    """
    weekdays = ['Monday', 'Tuesday', 'Wednesday', 'Thursday',
                'Friday', 'Saturday', 'Sunday']

    if end_date is None:
        end_date = datetime.today()
    day_num = end_date.weekday()  # 0~6 一~日
    day_num_target = weekdays.index(dayname)  # 对应list索引
    days_ago = (7 + day_num - day_num_target) % 7
    if days_ago == 0:
        days_ago = 7
    target_date = end_date - timedelta(days=days_ago)
    return target_date


def get_month_range(start_date=None):
    """获取当前日期范围"""
    print('当前日期:', date.today())
    if start_date is None:
        start_date = date.today().replace(day=1)  # 将日替换成1
    _, days_in_month = calendar.monthrange(start_date.year, start_date.month)  # 返回两个整数组成的元组，第一个是该月的第一天是星期几，第二个是该月的天数。
    end_date = start_date + timedelta(days=days_in_month)  # 生成一个时间差
    print('本月1号与下月1号', start_date, end_date)
    return (start_date, end_date)


if __name__ == '__main__':
    print(get_previous_day_by_week('Monday'))
    print(get_previous_day_by_week('Sunday', datetime(2012, 12, 21)))
    print(get_month_range())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2022/12/8 0:50
# @Author : xiang
# @File : time_type
# @Project : caishengxiang
"""
描述：时间类型转换
"""
import time
from datetime import datetime


def str_to_timestamp(time_str: str, time_format="%Y-%m-%d %H:%M:%S") -> float:
    """字符串转时间戳"""
    stamp = time.mktime(time.strptime(time_str, time_format))
    return stamp


def timestamp_to_str(stamp: float, time_format="%Y-%m-%d %H:%M:%S") -> str:
    """时间戳转字符串"""
    time_str = time.strftime(time_format, time.localtime(stamp))
    return time_str


def datetime_to_str(dtime: datetime, time_format="%Y-%m-%d %H:%M:%S") -> str:
    """datetime转字符串"""
    time_str = dtime.strftime(time_format)
    return time_str


def str_to_datetime(time_str: str, time_format="%Y-%m-%d %H:%M:%S") -> datetime:
    """字符串转datetime"""
    dtime = datetime.strptime(time_str, time_format)
    return dtime


def datetime_to_timestamp(dtime: datetime):
    """datetime转时间戳"""
    stamp = time.mktime(dtime.timetuple())
    return stamp


def timestamp_to_datetime(stamp: float) -> datetime:
    """时间戳转datetime"""
    dtime = datetime.fromtimestamp(stamp)
    return dtime


if __name__ == '__main__':
    print(str_to_timestamp('2018-08-07 15:40:30'))
    print(timestamp_to_str(time.time()))
    print(datetime_to_str(datetime.now()))
    print(str_to_datetime('2018-08-07 15:40:30'))
    print(datetime_to_timestamp(datetime.now()))
    print(timestamp_to_datetime(time.time()))



# ---encoding:utf-8---
# @Time    : 2023/7/18 00:13
# @Author  : caishengxiang
# @Email   : 383301857@qq.com
# @File    : gpt.py
# @Project : caishengxiang
# 导入revChatGPT.V1中的Chatbot类
# ppython -m pip install --upgrade revChatGPT
import json
from revChatGPT.V1 import Chatbot

# 创建Chatbot实例并传入config参数，包括登录OpenAI的账户信息
# json.load(open('./gpt.json')
chatbot = Chatbot(config={"email": "yickho13@gmail.com",
    "password": "Heyi198401"})
# 定义问题
question = """使用达芬奇16做专场视频剪辑
"""

# 输出问题
print(question)

# 输出ChatGPT的回答
print("ChatGPTBot: ")

# 定义prev_text变量，用于保存上一次对话的文本内容
prev_text = ""

# 通过ask方法向ChatGPT发送问题并获取回答
for data in chatbot.ask(question):
    # 从回答数据中提取ChatGPT的回答，并去除前面已经输出过的文本部分
    message = data["message"][len(prev_text):]

    # 输出ChatGPT的回答
    print(message, end="", flush=True)

    # 更新prev_text变量
    prev_text = data["message"]

# 输出空行，以便下一轮对话
print()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2022/12/8 0:32
# @Author : xiang
# @File : c_dateutil
# @Project : caishengxiang
"""
描述：
对大多数基本的日期和时间处理问题， datetime 模块已经足够了。
如果你需要执行更加复杂的日期操作，
比如处理时区，模糊时间范围，节假日计算等等， 可以考虑使用 dateutil模块

许多类似的时间计算可以使用 dateutil.relativedelta() 函数代替。 但是，有一点需要注意的就是，
它会在处理月份(还有它们的天数差距)的时候填充间隙。看例子最清楚：
"""

try:
    from dateutil.relativedelta import relativedelta
except:
    print("pip install python-dateutil")
import datetime


def my_dateutil():
    a = datetime.datetime(2012, 9, 23)
    # print(a + timedelta(months=1))
    print(a + relativedelta(months=+1))
    print(a + relativedelta(months=+4))

    b = datetime.datetime(2012, 12, 21)
    d = b - a
    print(d)
    d = relativedelta(b, a)
    print(d)
    print(type(d))
    print(d.months)
    print(d.days)


if __name__ == '__main__':
    my_dateutil()



# -*-coding:utf-8-*-
import os
import zipfile
from caishengxiang.libs.zipstream import ZipStream


def get_zip_stream(dirpath, chunksize=1024 * 10):
    """
    根据实际目录路径 生成流下载对象
    """
    files = []
    parent_dir = os.path.dirname(dirpath)  # 压缩文件夹的父目录
    for path, dirnames, filenames in os.walk(dirpath):  # 遍历所有子目录
        # 去掉目标根路径，只对目标文件夹下边的文件及文件夹进行压缩
        z_dir_path = path.replace(parent_dir, '')  # 去掉父目录 获取相对目录
        for filename in filenames:
            file_path = os.path.join(path, filename)  # 文件在外面的实际路径
            z_file_path = os.path.join(z_dir_path, filename)  # 在zip内的相对路径
            files.append({'file': file_path, 'name': z_file_path})
    zs = ZipStream(files, chunksize=chunksize)
    return zs


def zip_dir(dirpath, out_path):
    """
    压缩指定文件夹 到指定路径.zip
    :param dirpath: 文件夹路径 dirpath : '/home/xiang/workproject/jpt_filesystem/static/log'
    :param out_path: 导出路径 '/home/xiang/workproject/jpt_filesystem/tests/log.zip'
    :return: 无
    """
    _diranme = os.path.dirname(dirpath)
    dir_dir_path = os.path.dirname(dirpath)  # 压缩文件夹的父目录

    zip = zipfile.ZipFile(out_path, "w", zipfile.ZIP_DEFLATED)
    for path, dirnames, filenames in os.walk(dirpath):  # 遍历所有子目录
        # 去掉目标根路径，只对目标文件夹下边的文件及文件夹进行压缩
        z_dir_path = path.replace(dir_dir_path, '')  # 去掉父目录 获取相对目录
        for filename in filenames:
            file_path = os.path.join(path, filename)  # 文件在外面的实际路径
            z_file_path = os.path.join(z_dir_path, filename)  # 在zip内的相对路径
            zip.write(file_path, z_file_path)
    zip.close()
    return out_path


def zip_dir_bytes(dirpath, zip_bytes_iO):
    """
    压缩指定文件夹 到内存
    :param dirpath:实际文件夹路径 dirpath : '/home/xiang/workproject/jpt_filesystem/static/log'
    :param zip_bytes_iO: BytesIO  例子  memory_file = BytesIO()
    :return: BytesIO
    """
    _diranme = os.path.dirname(dirpath)
    dir_dir_path = os.path.dirname(dirpath)  # 压缩文件夹的父目录

    zip = zipfile.ZipFile(zip_bytes_iO, "w", zipfile.ZIP_DEFLATED)
    for path, dirnames, filenames in os.walk(dirpath):  # 遍历所有子目录
        # 去掉目标根路径，只对目标文件夹下边的文件及文件夹进行压缩
        z_dir_path = path.replace(dir_dir_path, '')  # 去掉父目录 获取相对目录
        for filename in filenames:
            file_path = os.path.join(path, filename)  # 文件在外面的实际路径
            z_file_path = os.path.join(z_dir_path, filename)  # 在zip内的相对路径
            with open(file_path, 'rb') as fp:
                content = fp.read()
                print(z_file_path)
                zip.writestr(z_file_path, content)
    zip.close()
    return zip_bytes_iO


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time : 2022/12/8 21:45
# @Author : xiang
# @File : classs
# @Project : caishengxiang
"""
描述：
"""


def slots_example():
    """
    当一个类需要创建大量实例时，可以通过__slots__声明实例所需要的属性，

例如，class Foo(object): __slots__ = ['foo']。这样做带来以下优点：

更快的属性访问速度
减少内存消耗

    默认情况下，访问一个实例的属性是通过访问该实例的__dict__来实现的。如访问a.x就相当于访问a.__dict__['x']
       1 a.x 2. a.__dict__ 3. a.__dict__['x'] 4. 结果
    定义了__slots__的类会为每个属性创建一个描述器。访问属性时就直接调用这个描述器。在这里我将它拆分为三步：
      1 b.x 2. member decriptor 3. 结果
    :return:
    """

    class Member:
        """生成限定属性的对象"""
        __slots__ = ['name', 'value']

    m = Member()
    m.name = 'csx'
    print(m.name)
    try:
        m.xx = 'xx'
    except Exception as e:
        print('报错就对了')


from abc import ABCMeta, abstractmethod


class Father(metaclass=ABCMeta):

    @abstractmethod
    def hello(self, name):
        pass


class Sub(Father):
    pass


def abc_example():
    try:
        Sub()
    except Exception as e:
        print('会提示必须要hello方法:{}'.format(e))


if __name__ == '__main__':
    slots_example()
    abc_example()



# ---encoding:utf-8---
# @Time    : 2023/7/18 00:31
# @Author  : caishengxiang
# @Email   : 383301857@qq.com
# @File    : gptV3.py
# @Project : caishengxiang
api_key = "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"

from revChatGPT.V3 import Chatbot

chatbot = Chatbot(api_key=api_key)
for data in chatbot.ask_stream("如何使用达芬奇17做专场视频剪辑？"):
    print(data, end="", flush=True)



# ---encoding:utf-8---
# @Time    : 2023/7/9 19:12
# @Author  : caishengxiang
# @Email   : 383301857@qq.com
# @File    : finds.py
# @Project : caishengxiang

import re


def find_email(strs):
    pattern = re.compile(r"[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(?:\.[a-zA-Z0-9_-]+)")
    result = pattern.findall(strs)
    return result


def find_id_card(strs):
    pattern = re.compile(
        r"[1-9]\d{5}(?:18|19|(?:[23]\d))\d{2}(?:(?:0[1-9])|(?:10|11|12))(?:(?:[0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]")
    result = pattern.findall(strs)
    return result


def find_china_phone(strs):
    pattern = re.compile(r"1[356789]\d{9}")
    result = pattern.findall(strs)
    return result


def find_domain(strs):
    """查找域名"""
    pattern = re.compile(r"(?:(?:http:\/\/)|(?:https:\/\/))?(?:[\w](?:[\w\-]{0,61}[\w])?\.)+[a-zA-Z]{2,6}(?:\/)")
    result = pattern.findall(strs)
    return result


def find_ipv4(strs):
    pattern = re.compile(r"((?:(?:25[0-5]|2[0-4]\d|[01]?\d?\d)\.){3}(?:25[0-5]|2[0-4]\d|[01]?\d?\d))")
    result = pattern.findall(strs)
    return result


def find_date(strs):
    pattern = re.compile(r"\d{4}(?:-|\/|.)\d{1,2}(?:-|\/|.)\d{1,2}")
    result = pattern.findall(strs)
    return result


if __name__ == '__main__':
    print(find_email(strs='我的私人邮箱是zhuwjwh@outlook.com，公司邮箱是123456@qq.org，麻烦登记一下？'))
    print(find_id_card(strs='小明的身份证号码是342623198910235163，手机号是13987692110'))
    print(find_china_phone(strs='小明的手机号是13987692110，你明天打给他'))
    print(find_domain(strs='Python官网的网址是https://www.python.org/'))
    print(find_ipv4(
        strs="""请输入合法IP地址，非法IP地址和其他字符将被过滤！
增、删、改IP地址后，请保存、关闭记事本！
192.168.8.84
192.168.8.85
192.168.8.86
0.0.0.1
256.1.1.1
192.256.256.256
192.255.255.255
aa.bb.cc.dd"""
    ))
    print(find_date(strs='今天是2020/12/20，去年的今天是2019.12.20，明年的今天是2021-12-20'))



#
# ZIP File streaming
# based on official ZIP File Format Specification version 6.3.4
# https://pkware.cachefly.net/webdocs/casestudies/APPNOTE.TXT
#
import os
import time
import zlib
from . import consts


__all__ = ("ZipStream", )


class Processor:
    def __init__(self, file_struct):
        self.crc = 0
        self.o_size = self.c_size = 0
        if file_struct['cmethod'] is None:
            self.process = self._process_through
            self.tail = self._no_tail
        elif file_struct['cmethod'] == 'deflate':
            self.compr = zlib.compressobj(5, zlib.DEFLATED, -15)
            self.process = self._process_deflate
            self.tail = self._tail_deflate

    # no compression
    def _process_through(self, chunk):
        self.o_size += len(chunk)
        self.c_size = self.o_size
        self.crc = zlib.crc32(chunk, self.crc)
        return chunk

    def _no_tail(self):
        return b''

    # deflate compression
    def _process_deflate(self, chunk):
        self.o_size += len(chunk)
        self.crc = zlib.crc32(chunk, self.crc)
        chunk = self.compr.compress(chunk)
        self.c_size += len(chunk)
        return chunk

    def _tail_deflate(self):
        chunk = self.compr.flush(zlib.Z_FINISH)
        self.c_size += len(chunk)
        return chunk

    # after processing counters and crc
    def state(self):
        return self.crc, self.o_size, self.c_size


class ZipBase:

    def __init__(self, files=[], chunksize=1024):
        """
        files - list of files, or generator returning files
                each file entry should be represented as dict with
                parameters:
                file - full path to file name
                name - (optional) name of file in zip archive
                       if not used, filename stripped from 'file' will be used
                stream - (optional) can be used as replacement for 'file'
                         entry, will be treated as generator returnig
                         chunks of data that will be streamed in archive.
                         If used, then 'name' entry is required.
        chunksize - default size of data block streamed from files
        """
        self._source_of_files = files
        self.__files = []
        self.__version = consts.ZIP32_VERSION
        self.zip64 = False
        self.chunksize = chunksize
        # this flag tuns on signature for data descriptor record.
        # see section 4.3.9.3 of ZIP File Format Specification
        self.__use_ddmagic = True
        # central directory size and placement
        self.__cdir_size = self.__offset = 0

    def zip64_required(self):
        """
        Turn on zip64 mode for archive
        """
        raise NotImplementedError("Zip64 is not supported yet")

    def _create_file_struct(self, data):
        """
        extract info about streamed file and return all processed data
        required in zip archive
        """
        # date and time of file
        dt = time.localtime()
        dosdate = ((dt[0] - 1980) << 9 | dt[1] << 5 | dt[2]) \
            & 0xffff
        dostime = (dt[3] << 11 | dt[4] << 5 | (dt[5] // 2)) \
            & 0xffff

        # check zip32 limit
        # stats = os.stat(data['file'])
        #  if stats.st_size > consts.ZIP32_LIMIT:
        #     self.zip64_required()
        # file properties used in zip
        file_struct = {'mod_time': dostime,
                       'mod_date': dosdate,
                       'crc': 0,  # will be calculated during data streaming
                       "offset": 0,  # file header offset in zip file
                       'flags': 0b00001000}  # flag about using data descriptor is always on

        if 'file' in data:
            file_struct['src'] = data['file']
            file_struct['stype'] = 'f'
        elif 'stream' in data:
            file_struct['src'] = data['stream']
            file_struct['stype'] = 's'
        else:
            raise Exception('No file or stream in sources')

        cmpr = data.get('compression', None)
        if cmpr not in (None, 'deflate'):
            raise Exception('Unknown compression method %r' % cmpr)
        file_struct['cmethod'] = cmpr
        file_struct['cmpr_id'] = {
            None: consts.COMPRESSION_STORE,
            'deflate': consts.COMPRESSION_DEFLATE}[cmpr]

        # file name in archive
        if 'name' not in data:
            data['name'] = os.path.basename(data['file'])
        try:
            file_struct['fname'] = data['name'].encode("ascii")
        except UnicodeError:
            file_struct['fname'] = data['name'].encode("utf-8")
            file_struct['flags'] |= consts.UTF8_FLAG
        return file_struct

    # zip structures creation

    def _make_extra_field(self, signature, data):
        """
        Extra field for file
        """
        fields = {"signature": signature,
                  "size": len(data)}
        head = consts.EXTRA_TUPLE(**fields)
        head = consts.EXTRA_STRUCT.pack(*head)
        return head + data

    def _make_local_file_header(self, file_struct):
        """
        Create file header
        """
        fields = {"signature": consts.LF_MAGIC,
                  "version": self.__version,
                  "flags": file_struct['flags'],
                  "compression": file_struct['cmpr_id'],
                  "mod_time": file_struct['mod_time'],
                  "mod_date": file_struct['mod_date'],
                  "crc": 0,
                  "uncomp_size": 0,
                  "comp_size": 0,
                  "fname_len": len(file_struct['fname']),
                  "extra_len": 0}
        head = consts.LF_TUPLE(**fields)
        head = consts.LF_STRUCT.pack(*head)
        head += file_struct['fname']
        return head

    def _make_data_descriptor(self, file_struct, crc, org_size, compr_size):
        """
        Create file descriptor.
        This function also updates size and crc fields of file_struct
        """
        # hack for making CRC unsigned long
        file_struct['crc'] = crc & 0xffffffff
        file_struct['size'] = org_size
        file_struct['csize'] = compr_size
        fields = {"uncomp_size": file_struct['size'],
                  "comp_size": file_struct['csize'],
                  "crc": file_struct['crc']}
        descriptor = consts.DD_TUPLE(**fields)
        descriptor = consts.DD_STRUCT.pack(*descriptor)
        if self.__use_ddmagic:
            descriptor = consts.DD_MAGIC + descriptor
        return descriptor

    def _make_cdir_file_header(self, file_struct):
        """
        Create central directory file header
        """
        fields = {"signature": consts.CDFH_MAGIC,
                  "system": 0x03,  # 0x03 - unix
                  "version": self.__version,
                  "version_ndd": self.__version,
                  "flags": file_struct['flags'],
                  "compression": file_struct['cmpr_id'],
                  "mod_time": file_struct['mod_time'],
                  "mod_date": file_struct['mod_date'],
                  "uncomp_size": file_struct['size'],
                  "comp_size": file_struct['csize'],
                  "offset": file_struct['offset'],  # < file header offset
                  "crc": file_struct['crc'],
                  "fname_len": len(file_struct['fname']),
                  "extra_len": 0,
                  "fcomm_len": 0,  # comment length
                  "disk_start": 0,
                  "attrs_int": 0,
                  "attrs_ext": 0}
        cdfh = consts.CDLF_TUPLE(**fields)
        cdfh = consts.CDLF_STRUCT.pack(*cdfh)
        cdfh += file_struct['fname']
        return cdfh

    def _make_cdend(self):
        """
        make end of central directory record
        """
        fields = {"signature": consts.CD_END_MAGIC,
                  "disk_num": 0,
                  "disk_cdstart": 0,
                  "disk_entries": len(self.__files),
                  "total_entries": len(self.__files),
                  "cd_size": self.__cdir_size,
                  "cd_offset": self._offset_get(),
                  "comment_len": 0}
        cdend = consts.CD_END_TUPLE(**fields)
        cdend = consts.CD_END_STRUCT.pack(*cdend)
        return cdend

    def _make_end_structures(self):
        """
        cdir and cdend structures are saved at the end of zip file
        """
        # stream central directory entries
        for idx, file_struct in enumerate(self.__files):
            chunk = self._make_cdir_file_header(file_struct)
            self.__cdir_size += len(chunk)
            yield chunk
        # stream end of central directory
        yield self._make_cdend()

    def _offset_add(self, value):
        self.__offset += value

    def _offset_get(self):
        return self.__offset

    def _add_file_to_cdir(self, file_struct):
        self.__files.append(file_struct)

    def _cleanup(self):
        """
        Clean all structs after streaming
        """
        self.__files = []
        self.__cdir_size = self.__offset = 0


class ZipStream(ZipBase):

    def data_generator(self, src, src_type):
        if src_type == 's':
            for chunk in src:
                yield chunk
            return
        if src_type == 'f':
            with open(src, "rb") as fh:
                while True:
                    part = fh.read(self.chunksize)
                    if not part:
                        break
                    yield part
            return

    def _stream_single_file(self, file_struct):
        """
        stream single zip file with header and descriptor at the end
        """
        yield self._make_local_file_header(file_struct)
        pcs = Processor(file_struct)
        for chunk in self.data_generator(file_struct['src'], file_struct['stype']):
            chunk = pcs.process(chunk)
            if len(chunk) > 0:
                yield chunk
        chunk = pcs.tail()
        if len(chunk) > 0:
            yield chunk
        yield self._make_data_descriptor(file_struct, *pcs.state())

    def stream(self):
        """
        Stream complete archive
        """
        # stream files
        for idx, source in enumerate(self._source_of_files):
            file_struct = self._create_file_struct(source)
            # file offset in archive
            file_struct['offset'] = self._offset_get()
            self._add_file_to_cdir(file_struct)
            # file data
            for chunk in self._stream_single_file(file_struct):
                self._offset_add(len(chunk))
                yield chunk
        # stream zip structures
        for chunk in self._make_end_structures():
            yield chunk
        self._cleanup()



from .zipstream import ZipStream
import sys

# AioZipStream is avilable from Python 3.6 version
if (sys.version_info.major >= 3) and (sys.version_info.minor >= 6):
    from .aiozipstream import AioZipStream
del sys

version = "0.4"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/7/19 上午12:32
# @Author  : caishengxiang
# @File    : simple_log.py
"""
简易日志
"""
import os
import time
import datetime
import threading
from functools import reduce
import pathlib


class SimpleLog:

    def __init__(self, log_dir, logger_level='debug', backup_days=90, suffix='.log'):
        if not os.path.exists(log_dir):
            pathlib.Path(log_dir).mkdir(parents=True, exist_ok=True)  # 创建目录
        self.level_relations = {
            'debug': 10,
            'info': 20,
            'warning': 30,
            'error': 40,
            'crit': 50
        }
        self.logger_level = logger_level.lower()
        self.logger_level_num = self.level_relations[logger_level]
        self.log_dir = log_dir
        self.backup_days = backup_days
        self.suffix = suffix
        self.hostname = os.getenv('HOSTNAME') or os.getenv('hostname')
        try:
            self.pid = os.getpid()
        except:
            self.pid = 'None'

    def _clean_logs(self):
        """清理过期日志"""
        try:
            file_names = os.listdir(self.log_dir)
            filter_file_names = filter(lambda file_name: file_name.endswith(self.suffix), file_names)
            for file_name in filter_file_names:
                date_str = file_name[:-len(self.suffix)]
                file_time = datetime.datetime.strptime(date_str, '%Y-%m-%d')
                clean_time = datetime.datetime.now() - datetime.timedelta(days=self.backup_days)
                if file_time < clean_time:
                    try:
                        os.remove(os.path.join(self.log_dir, file_name))
                    except Exception as e:
                        print(e)
                        continue
        except Exception as e:
            print(e)

    def _write_log(self, *contents, logger_level=None):
        if logger_level is None:
            logger_level = self.logger_level
        try:
            if not os.path.exists(self.log_dir):
                pathlib.Path(self.log_dir).mkdir(parents=True, exist_ok=True)
            date = time.strftime('%Y-%m-%d')
            filename = '{}{}'.format(str(date), self.suffix)
            file = os.path.join(self.log_dir, filename)
            if not os.path.exists(file):
                self._clean_logs()

            content = str(reduce(lambda x, y: str(x) + '  ' + str(y), contents))
            fp = open(file, 'a', encoding='utf-8')
            now_str = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            hostname_str = ''
            if self.hostname:
                hostname_str = '[hostname:{hostname}]'.format(hostname=self.hostname)
            log_header = "[{now}]{hostname_str}[pid:{pid}][{logger_level}]".format(now=now_str,
                                                                                   hostname_str=hostname_str,
                                                                                   pid=self.pid,
                                                                                   logger_level=logger_level)
            strlog = log_header + content + '\n'
            print(strlog)
            fp.write(strlog)
            fp.close()
        except Exception as e:
            print(e)

    def write_log(self, *contents, **kwargs):
        try:
            t = threading.Thread(target=self._write_log, args=contents, kwargs=kwargs)
            t.start()
        except Exception as e:
            print(e)

    def debug(self, *contents):
        if self.logger_level_num <= self.level_relations['debug']:
            self.write_log(*contents, logger_level='debug')

    def info(self, *contents):
        if self.logger_level_num <= self.level_relations['info']:
            self.write_log(*contents, logger_level='info')

    def warning(self, *contents):
        if self.logger_level_num <= self.level_relations['warning']:
            self.write_log(*contents, logger_level='warning')

    def error(self, *contents):
        if self.logger_level_num <= self.level_relations['error']:
            self.write_log(*contents, logger_level='error')

    def crit(self, *contents):
        if self.logger_level_num <= self.level_relations['crit']:
            self.write_log(*contents, logger_level='crit')


if __name__ == '__main__':
    test_logger = SimpleLog(log_dir='/tmp/simple_log', logger_level='debug', backup_days=90, suffix='.test_log')
    test_logger.debug('debug', 'xx')
    test_logger.info('info', 9, 'zzzz')
    test_logger.warning('warning', 10, 'xxx')



# -*-coding:utf-8-*-
"""
pip install paramiko
pip install pyecharts

如需监控网络 服务器：
sudo yum install epel-release
sudo yum install nload
"""
import time
import logging
import os
import datetime
import re
import pathlib
from concurrent.futures import ThreadPoolExecutor

try:
    import paramiko
except:
    raise Exception('you need: pip install paramiko')

from caishengxiang.utils.draw_tools.echarts_draw import Draw
from pyecharts.charts import Page


def exec_ssh_command(hostname, username, password, cmd='top -bn 1'):
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(hostname=hostname, username=username, password=password)
    stdin, stdout, stderr = ssh.exec_command(cmd)
    output = stdout.read()
    content = output.decode()
    ssh.close()
    return content


class Monitor:
    def __init__(self, hostname, username, password, save_dir="./monitors"):
        self.hostname = hostname
        self.username = username
        self.password = password
        self.save_dir = os.path.abspath(save_dir)
        pathlib.Path(self.save_dir).mkdir(parents=True, exist_ok=True)
        self.monitor_map = dict()
        self.page = Page()
        self.thread_pool = ThreadPoolExecutor(max_workers=10)

    def _exec_ssh_command(self, command: str = 'top -bn 1'):
        return exec_ssh_command(self.hostname, self.username, self.password, command)

    def default_time_handle(self):
        return str(datetime.datetime.now())

    def default_content_handle(self, content: str):
        content = content.strip()
        return float(content)

    def add_monitor(self, command, content_handle=None, time_handle=None, y_name='使用率%',
                    x_name='时间', title='cpu监控'):
        if self.monitor_map.get(title):
            raise Exception('已经存在同名监控:{}'.format(title))
        self.monitor_map[title] = {
            'command': command,
            'content_handle': content_handle if content_handle else self.default_content_handle,
            'time_handle': time_handle if time_handle else self.default_time_handle,
            'draw': Draw(y_name=y_name, x_name=x_name,
                         title='{hostname}:{title}'.format(hostname=self.hostname, title=title),
                         save_path=os.path.join(self.save_dir,
                                                '{hostname}_{title}.html'.format(hostname=self.hostname, title=title)))
        }

    def _look(self, command, draw, content_handle, time_handle):
        try:
            content = self._exec_ssh_command(command)
            y_data = content_handle(content)
            x_data = time_handle()
            draw.add_data(x_data=x_data, y_data=y_data)
            draw.save()
        except Exception as e:
            logging.error('command:{} error:{}'.format(command, e))

    def looks(self):
        for title, monitor_dict in self.monitor_map.items():
            command = monitor_dict['command']
            content_handle = monitor_dict['content_handle']
            time_handle = monitor_dict['time_handle']
            draw = monitor_dict['draw']
            self.thread_pool.submit(self._look, command, draw, content_handle, time_handle)

    def end(self):
        for title, monitor_dict in self.monitor_map.items():
            draw = monitor_dict['draw']
            self.page.add(draw.draw())
        self.page.render(os.path.join(self.save_dir, '{}_总监控.html'.format(self.hostname)))

    def start_server(self, host='0.0.0.0', port=8888):
        """启动监控web服务"""
        pass

if __name__ == '__main__':
    def top_cpu_usage(content):
        """
        return 使用率(%)
        """
        lines = content.split('\n')
        cpu_line = [x for x in lines if 'Cpu(s)' in x][0]
        cpu_strs = cpu_line.split(':')[1].split(',')
        cpu_id_str = cpu_strs[3].strip()
        cpu_id = float(cpu_id_str.split()[0])
        usage = 100 - cpu_id
        return round(usage, 2)


    def top_memory_usage(content):
        """
        return 字节
        """
        lines = content.split('\n')
        mem_line = [x for x in lines if x.startswith('KiB Mem')][0]
        mem_infos = mem_line.split(',')
        used = [x for x in mem_infos if 'used' in x][0]
        used = used.split()[0]
        # return int(used) # 字节
        return round(used / 1024, 2)  # MiB
        # return round(used / (1024 * 1024), 2) # GiB


    # from pyecharts.globals import CurrentConfig, OnlineHostType
    #
    # dir_path = os.path.dirname(__file__)
    # assets_path = os.path.join(dir_path, 'assets')
    # CurrentConfig.ONLINE_HOST = assets_path + '/'

    monitor1 = Monitor(hostname='192.168.0.111', username='csx', password='Orange_123', save_dir="D:\\tmp\\monitor")
    monitor1.add_monitor(command="free -m | sed -n '2p'|awk '{print $3/$2*100}'", y_name='内存使用率%', title='内存监控')
    monitor1.add_monitor(command="top -bn 1", content_handle=top_cpu_usage, y_name='cpu使用率%', title='cpu监控')

    monitor2 = Monitor(hostname='192.168.0.112', username='csx', password='Orange_123', save_dir="D:\\tmp\\monitor")
    monitor2.add_monitor(command="free -m | sed -n '2p'|awk '{print $3/$2*100}'", y_name='内存使用率%', title='内存监控')
    monitor2.add_monitor(command="top -bn 1", content_handle=top_cpu_usage, y_name='cpu使用率%', title='cpu监控')

    monitor3 = Monitor(hostname='192.168.0.108', username='csx', password='Orange_123', save_dir="D:\\tmp\\monitor")
    monitor3.add_monitor(command="free -m | sed -n '2p'|awk '{print $3/$2*100}'", y_name='内存使用率%', title='内存监控')
    monitor3.add_monitor(command="top -bn 1", content_handle=top_cpu_usage, y_name='cpu使用率%', title='cpu监控')

    look_time = 100  # 秒
    for i in range(look_time):
        monitor1.looks()
        monitor2.looks()
        monitor3.looks()
        time.sleep(0.1)
    monitor1.end()
    monitor2.end()
    monitor3.end()



# -*-coding:utf-8-*-
import os
import sys
import subprocess
import re
import platform
import ctypes


def is_writable(path):
    """
    是否可写
    :param path：实际地址
    :return:
    """
    return os.access(path, os.W_OK)


def is_readable(path):
    """
    是否可读
    :param path: 实际地址
    :return:
    """
    return os.access(path, os.R_OK)


def is_executable(path):
    """
    是否可执行
    :param path: 实际地址
    :return:
    """
    return os.access(path, os.X_OK)

def get_path_size_by_linux(path: str):
    """
    利用 linux 原生命令 查看文件(夹)大小
    @param path:
    @return: @int size 字节
    """

    def du_sb():
        context = None
        size = None
        cmds = ['du', '-sb', path]
        p = subprocess.Popen(cmds, stdout=subprocess.PIPE)
        out, err = p.communicate()
        for line in out.splitlines():
            context = line.decode()

        if context:
            pattern = re.compile(r'^(\d+)')
            searchObj = pattern.search(context)
            size_str = searchObj.group(1)
            size = int(size_str)
        return size

    def du_sk():
        context = None
        size = None
        cmds = ['du', '-sk', path]
        p = subprocess.Popen(cmds, stdout=subprocess.PIPE)
        out, err = p.communicate()
        for line in out.splitlines():
            context = line.decode()

        if context:
            pattern = re.compile(r'^(\d+)')
            searchObj = pattern.search(context)
            size_str = searchObj.group(1)
            size = int(size_str)
        return size * 1024  # kb转b

    size = du_sb()
    if size is None:
        size = du_sk()
    return size


def get_path_size(path):
    """
    根据路径获取 文件/文件夹的大小
    :param path:
    :return: dir_size 字节
    """
    if os.path.isfile(path):
        return os.path.getsize(path)
    if 'linux' in sys.platform:  # linux原生命令性能更好
        _size = get_path_size_by_linux(path)
        if _size:
            return _size
    dir_size = 0
    for _path, dirnames, filenames in os.walk(path):  # 遍历所有子目录
        for filename in filenames:
            file_path = os.path.join(_path, filename)
            try:
                size = os.path.getsize(file_path)
            except:
                size = 0
            dir_size += size
    return dir_size


def get_suffix(file_name: str) -> str:
    """获取文件后缀"""
    suffix = ''
    split = file_name.split(".")
    if len(split) > 1:
        suffix = split[-1]
    return suffix


def check_file_too_large(path, max_size):
    """
    检查文件（夹）大小
    :param path:
    :param max_size:
    :return:
    """
    if not os.path.exists(path):
        return False
    if os.path.isfile(path):
        if os.path.getsize(path) > max_size:
            return True
    size = 0
    for path, dirnames, filenames in os.walk(path):
        # 递归查询文件大小
        for f in filenames:
            _path = os.path.join(path, f)
            try:
                _size = os.path.getsize(_path)
            except:
                _size = 0
            size += _size
            print(size)
            if size > max_size:
                return True


def check_file_too_many(path, max_num):
    """检查文件数量 （非常节约性能）"""
    if not os.path.exists(path):
        return False
    if os.path.isfile(path):
        return False
    num = 0
    for path, dirnames, filenames in os.walk(path):
        num += len(filenames)
        if num > max_num:
            return True



def get_free_space_mb(folder):
    """
    获取当前目录所在磁盘 剩余空间
    :param folder: 磁盘路径 例如 D:\\
    :return: 剩余空间 单位 G
    """
    if platform.system() == 'Windows':
        free_bytes = ctypes.c_ulonglong(0)
        ctypes.windll.kernel32.GetDiskFreeSpaceExW(ctypes.c_wchar_p(folder), None, None, ctypes.pointer(free_bytes))
        return free_bytes.value / 1024
    else:
        st = os.statvfs(folder)
        return st.f_bavail * st.f_frsize  # 非超级用户可获得的块数×分栈大小==磁盘剩余空间


def get_blocks_space_mb(folder):
    """获取当前目录所在磁盘 全部空间"""
    st = os.statvfs(folder)
    return st.f_blocks * st.f_frsize


def get_pvc_info(folder):
    """
    当前目录所在磁盘信息
    :param folder:
    :return: (剩余空间率, 剩余空间, 总空间)
    """
    free_size = get_free_space_mb(folder)  # 剩余空间
    total_size = get_blocks_space_mb(folder)
    free_space_rate = free_size / total_size
    return {'free_space_rate': free_space_rate, 'free_size': free_size, 'total_size': total_size}


# ---encoding:utf-8---
# @Time    : 2023/7/8 01:35
# @Author  : caishengxiang
# @Email   : 383301857@qq.com
# @File    : pssh_tool.py
# @Project : caishengxiang
"""

pip install parallel-ssh
"""
import logging

try:
    from pssh.clients import ParallelSSHClient
except Exception as e:
    print('you need pip install parallel-ssh, {}'.format(e))


def run_command(hosts, user, password, cmd, timeout=0, if_raise=False):
    client = ParallelSSHClient(hosts, user=user, password=password, timeout=timeout)
    output = client.run_command(cmd)
    client.pool.join()  # 等执行完
    for host_output in output:
        print("========== results of cmd:【{}】 in host:【{}】 =============".format(cmd, host_output.host))
        for line in host_output.stdout:
            print(line)

        exit_code = host_output.exit_code

        if exit_code != 0:
            errs = ""
            for err in host_output.stderr:
                errs += err
            print("\033[0;31;40m", '【{}】| {}'.format(host_output.host, errs), "\033[0m")
            # logging.error('【{}】| {}'.format(host_output.host, errs))
            if if_raise:
                raise Exception('【{}】| {}'.format(host_output.host, errs))


    del client


def run_commands(hosts, user, password, cmds, timeout=0, if_raise=False):
    for cmd in cmds:
        run_command(hosts=hosts, user=user, password=password, cmd=cmd, timeout=timeout, if_raise=if_raise)


if __name__ == '__main__':
    # run_command(hosts=["175.178.172.183"], user='ubuntu', password='xxxx', cmd='ls -a')
    # run_command(hosts=["175.178.172.183"], user='ubuntu', password='xxxx', cmd='rm ./xxxx')
    # run_command(hosts=["175.178.172.183"], user='ubuntu', password='xxxx', cmd='rm ./xxxx', if_raise=True)

    run_commands(hosts=["192.168.0.111"], user='csx', password='Orange_123',
                 cmds=['ls', 'ls'])



# -*-coding:utf-8-*-
import os
import pandas as pd

print(os.getcwd())

"""
all_df=pd.concat([df1,df2])
all_df=all_df.dropna(axis=0, how='any',subset=["Id"])
all_df["Id"]=all_df["Id"].astype(np.int64)
all_df.to_csv(all_df_path, index=False, sep=',')
"""


def delete_row_by_nan(df: pd.DataFrame, column) -> pd.DataFrame:
    """"""
    df = df.dropna(axis=0, how='any', subset=[column])
    return df



#
# ZIP File streaming
# based on official ZIP File Format Specification version 6.3.4
# https://pkware.cachefly.net/webdocs/casestudies/APPNOTE.TXT
#
import asyncio
from .zipstream import ZipBase, Processor
from concurrent import futures
try:
    import aiofiles
    aio_available = True
except ImportError:
    aio_available = False


__all__ = ("AioZipStream",)


class AioZipStream(ZipBase):
    """
    Asynchronous version of ZipStream
    """

    def __init__(self, *args, **kwargs):
        super(AioZipStream, self).__init__(*args, **kwargs)

    def __get_executor(self):
        # get thread pool executor
        try:
            return self.__tpex
        except AttributeError:
            self.__tpex = futures.ThreadPoolExecutor(max_workers=1)
            return self.__tpex

    async def _execute_aio_task(self, task, *args, **kwargs):
        # run synchronous task in separate thread and await for result
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(self.__get_executor(),
                                          task, *args, **kwargs)

    def _create_file_struct(self, data):
        if 'file' in data:
            if not aio_available:
                raise Exception(
                    "aiofiles module is required to stream files asynchronously")
        return super(AioZipStream, self)._create_file_struct(data)

    async def data_generator(self, src, src_type):
        if src_type == 's':
            async for chunk in src:
                yield chunk
            return
        if src_type == 'f':
            async with aiofiles.open(src, "rb") as fh:
                while True:
                    part = await fh.read(self.chunksize)
                    if not part:
                        break
                    yield part
            return

    async def _stream_single_file(self, file_struct):
        """
        stream single zip file with header and descriptor at the end
        """
        yield self._make_local_file_header(file_struct)
        pcs = Processor(file_struct)
        async for chunk in self.data_generator(file_struct['src'], file_struct['stype']):
            yield await self._execute_aio_task(pcs.process, chunk)
        chunk = await self._execute_aio_task(pcs.tail)
        # chunk = await pcs.aio_tail()
        if len(chunk) > 0:
            yield chunk
        yield self._make_data_descriptor(file_struct, *pcs.state())

    async def stream(self):
        # stream files
        for idx, source in enumerate(self._source_of_files):
            file_struct = self._create_file_struct(source)
            # file offset in archive
            file_struct['offset'] = self._offset_get()
            self._add_file_to_cdir(file_struct)
            # file data
            async for chunk in self._stream_single_file(file_struct):
                self._offset_add(len(chunk))
                yield chunk
        # stream zip structures
        for chunk in self._make_end_structures():
            yield chunk
        self._cleanup()
